#include "ModuleBase.h"
#include <algorithm>

Input::Input(std::string name, rapidjson::Value default_value, std::function<bool(rapidjson::Value &)> verification) {
    this->name= name;
    value= default_value;
}


std::vector<std::string> ModuleBase::getInputs(){
    std::vector<std::string> ret;
    for(auto& input  : inputs)
    {
        ret.push_back(input.name);
    }
    return ret;
}

int ModuleBase::addInput(std::string name, rapidjson::Value value,
                         std::function<bool(rapidjson::Value &)> verification){
    inputs.push_back(Input(name, value, verification));
    return inputs.size() - 1;
}

bool ModuleBase::connectInput(std::string input_name, std::shared_ptr<IOutput> output){
    std::vector<std::string> input_list = getInputs();
    auto it = std::find(input_list.begin(), input_list.end(), input_name);
    if(it == input_list.end()){
        return false;
    }

    int index = std::distance(input_list.begin(), it);
    if(inputs[index].verification(output->getValue())
    {
        inputs[index].output = output;
        return true;
    }
    return false;
}

rapidjson::Value& ModuleBase::getValue(int handle){
    if(inputs[handle].output == nullptr)
        return inputs[handle].value;

    return inputs[handle].output->getInternalValue();

}